#!/bin/bash

# Creates and sets up your Python 3.10 venv
python3.10 -m venv venv310
source venv310/bin/activate

# Make sure colcon doesn't try to index the venv310 directory
touch venv310/COLCON_IGNORE

# Install Python packages
pip install --upgrade pip
pip install \
    cython \
    pathlib2 \
    numpy \
    matplotlib \
    ipympl \
    jupyterlab \
    jupyterlab-widgets \
    ipykernel

pip3 install --upgrade pytest colcon-common-extensions setuptools

# Register venv310 as Jupyter kernel
python -m ipykernel install --user --name=venv310 --display-name="Python 3.10 (venv310)"

# Make sure the venv310 is activated when starting a new shell
echo "source /workspaces/ROS-Dynamic-Executor-Experiments/venv310/bin/activate" >> /root/.bashrc

echo "Setup complete! Activate with: source venv310/bin/activate"
echo "Then start JupyterLab with: jupyter lab --ip=0.0.0.0 --port=8888 --allow-root"
