#ifndef RTIS_PRIMES_WORKLOAD
#define RTIS_PRIMES_WORKLOAD

#include <thread>
#include <cstdio>

#include <time.h>
#include <sys/time.h>

// Parameters for the dummy load calibration
constexpr int TARGET_DURATION_MS = 100;
constexpr int MAX_ITERATIONS = 100;
constexpr int ACCEPTABLE_MARGIN_US = 500;

/**
 * @brief Dummy load calibration namespace.
 * This namespace contains functions and variables for calibrating the dummy load.
 * It uses an atomic integer to store the calibration factor.
 * The calibration factor is used to adjust the workload of the dummy load function.
 */
namespace DummyLoadCalibration
{
    inline std::atomic<int> calibration{1};

    inline void setCalibration(int const calib);

    inline int getCalibration();
}

/**
 * @brief Dummy load function to simulate a workload.
 * This function performs a busy wait for a specified duration.
 * It is used to calibrate the dummy load for the current system.
 */
void calibrate_dummy_load();

/**
 * @brief Dummy load function to simulate a workload.
 * This function performs a busy wait for a specified duration.
 * It is used to calibrate the dummy load for the current system.
 */
void dummy_load(int const load_ms);

#endif